#include <sta/rtos/thread.hpp>


#define IS_THREAD_SYS_FLAGS(flags)	( ( flags & STA_RTOS_THREAD_FLAGS_VALID_SYS_BITS ) == flags )
#define IS_THREAD_USER_FLAGS(flags)	( ( flags & STA_RTOS_THREAD_FLAGS_VALID_USER_BITS ) == flags )


namespace sta
{
	RtosThread::RtosThread()
		: handle_(nullptr)
	{}

	RtosThread::RtosThread(const Handle & handle)
		: handle_{handle}
	{}


	void RtosThread::notify(uint32_t flags)
	{
		STA_ASSERT(handle_.get() != nullptr);
		STA_ASSERT_MSG(IS_THREAD_USER_FLAGS(flags), "Only user flags allowed");

		osThreadFlagsSet(handle_.get(), flags);
	}

	void RtosThread::sysNotify(uint32_t flags)
	{
		STA_ASSERT(handle_.get() != nullptr);
		STA_ASSERT_MSG(IS_THREAD_SYS_FLAGS(flags), "Only system flags allowed");

		uint32_t rslt = osThreadFlagsSet(handle_.get(), flags);

		STA_ASSERT_MSG(rslt != osFlagsErrorUnknown, "Unknown thread flag error.");
		STA_ASSERT_MSG(rslt != osFlagsErrorParameter, "Parameter thread_id is not a valid thread or flags has highest bit set.");
		STA_ASSERT_MSG(rslt != osFlagsErrorResource, "The thread is in invalid state.");
	}

	uint32_t RtosThread::wait(uint32_t flags)
	{
		return osThreadFlagsWait(flags, osFlagsWaitAny, osWaitForever);
	}

	uint32_t RtosThread::clear(uint32_t flags)
	{
		uint32_t setFlags = osThreadFlagsClear(flags);

		STA_ASSERT(setFlags != (uint32_t)osError);

		return setFlags;
	}

	uint32_t RtosThread::getFlags()
	{
		STA_ASSERT(handle_.get() != nullptr);

		return osThreadFlagsGet();
	}

	void RtosThread::requestTermination()
	{
		// sysNotify(STA_RTOS_THREAD_FLAG_TERMINATE);
		terminate_ = true;
	}

	bool RtosThread::resetTerminationRequest()
	{
		bool temp = terminate_;
		terminate_ = false;

		return temp;
	}

	void RtosThread::terminate()
	{
		STA_ASSERT(handle_.get() != nullptr);
		osThreadTerminate(handle_.get());
	}
} // namespace sta
